require 'date'

module HelixWebServices
  # Create or edit Perforce user specifications and preferences.\n\nThere are three types of Perforce users: standard users, operator users, and service users.\nStandard users are the default, and each standard user consumes one Perforce license.\nThe operator user type is intended for system administrators; they are subject to the same restrictions on permissions as any other user, but are further restricted in that they can run only a limited subset of Perforce commands.\nService users are intended for inter-server communication in replicated and multi-server environments, and are restricted to an even smaller subset of Perforce commands.\nNeither operators nor service users consume Perforce licenses.
  class UserCommand
    # The Perforce username.
    attr_accessor :user

    # Type of user: standard, operator, or service.\n\nOnce you set the type, you cannot change it.
    attr_accessor :type

    # One of the following: perforce or ldap.\n\nSpecifying perforce enables authentication using Perforce’s internal db.user table or by way of an authentication trigger.\nThis is the default unless it is overridden with the auth.default.method configurable.\n\nSpecifying ldap enables authentication against AD/LDAP servers specified by the currently active LDAP configurations.
    attr_accessor :auth_method

    # The user’s email address. By default, this is user@client.
    attr_accessor :email

    # The date and time this specification was last updated.
    attr_accessor :update

    # The date and time this user last ran a Perforce command.
    attr_accessor :access

    # The user's full name.
    attr_accessor :full_name

    # Jobs matching this jobview appear on any changelists created by this user.\nJobs that are fixed by the changelist should be left in the changelist when it's submitted with p4 submit; other jobs should be deleted from the form before submission.
    attr_accessor :job_view

    # The user's password.
    attr_accessor :password

    # The date and time of the user’s last password change.\nIf the user has no password, this field is blank.
    attr_accessor :password_change

    # A list of files the user would like to review.\nThis field can include exclusionary mappings.
    attr_accessor :reviews

    # Attribute mapping from ruby-style variable name to JSON key.
    def self.attribute_map
      {
        
        :'user' => :'user',
        
        :'type' => :'type',
        
        :'auth_method' => :'authMethod',
        
        :'email' => :'email',
        
        :'update' => :'update',
        
        :'access' => :'access',
        
        :'full_name' => :'fullName',
        
        :'job_view' => :'jobView',
        
        :'password' => :'password',
        
        :'password_change' => :'passwordChange',
        
        :'reviews' => :'reviews'
        
      }
    end

    # Attribute type mapping.
    def self.swagger_types
      {
        :'user' => :'String',
        :'type' => :'String',
        :'auth_method' => :'String',
        :'email' => :'String',
        :'update' => :'DateTime',
        :'access' => :'DateTime',
        :'full_name' => :'String',
        :'job_view' => :'String',
        :'password' => :'String',
        :'password_change' => :'DateTime',
        :'reviews' => :'Array<String>'
        
      }
    end

    def initialize(attributes = {})
      return unless attributes.is_a?(Hash)

      # convert string to symbol for hash key
      attributes = attributes.inject({}){|memo,(k,v)| memo[k.to_sym] = v; memo}

      
      if attributes[:'user']
        self.user = attributes[:'user']
      end
      
      if attributes[:'type']
        self.type = attributes[:'type']
      end
      
      if attributes[:'authMethod']
        self.auth_method = attributes[:'authMethod']
      end
      
      if attributes[:'email']
        self.email = attributes[:'email']
      end
      
      if attributes[:'update']
        self.update = attributes[:'update']
      end
      
      if attributes[:'access']
        self.access = attributes[:'access']
      end
      
      if attributes[:'fullName']
        self.full_name = attributes[:'fullName']
      end
      
      if attributes[:'jobView']
        self.job_view = attributes[:'jobView']
      end
      
      if attributes[:'password']
        self.password = attributes[:'password']
      end
      
      if attributes[:'passwordChange']
        self.password_change = attributes[:'passwordChange']
      end
      
      if attributes[:'reviews']
        if (value = attributes[:'reviews']).is_a?(Array)
          self.reviews = value
        end
      end
      
    end

    # Check equality by comparing each attribute.
    def ==(o)
      return true if self.equal?(o)
      self.class == o.class &&
          user == o.user &&
          type == o.type &&
          auth_method == o.auth_method &&
          email == o.email &&
          update == o.update &&
          access == o.access &&
          full_name == o.full_name &&
          job_view == o.job_view &&
          password == o.password &&
          password_change == o.password_change &&
          reviews == o.reviews
    end

    # @see the `==` method
    def eql?(o)
      self == o
    end

    # Calculate hash code according to all attributes.
    def hash
      [user, type, auth_method, email, update, access, full_name, job_view, password, password_change, reviews].hash
    end

    # build the object from hash
    def build_from_hash(attributes)
      return nil unless attributes.is_a?(Hash)
      self.class.swagger_types.each_pair do |key, type|
        if type =~ /^Array<(.*)>/i
          if attributes[self.class.attribute_map[key]].is_a?(Array)
            self.send("#{key}=", attributes[self.class.attribute_map[key]].map{ |v| _deserialize($1, v) } )
          else
            #TODO show warning in debug mode
          end
        elsif !attributes[self.class.attribute_map[key]].nil?
          self.send("#{key}=", _deserialize(type, attributes[self.class.attribute_map[key]]))
        else
          # data not found in attributes(hash), not an issue as the data can be optional
        end
      end

      self
    end

    def _deserialize(type, value)
      case type.to_sym
      when :DateTime
        DateTime.parse(value)
      when :Date
        Date.parse(value)
      when :String
        value.to_s
      when :Integer
        value.to_i
      when :Float
        value.to_f
      when :BOOLEAN
        if value.to_s =~ /^(true|t|yes|y|1)$/i
          true
        else
          false
        end
      when /\AArray<(?<inner_type>.+)>\z/
        inner_type = Regexp.last_match[:inner_type]
        value.map { |v| _deserialize(inner_type, v) }
      when /\AHash<(?<k_type>.+), (?<v_type>.+)>\z/
        k_type = Regexp.last_match[:k_type]
        v_type = Regexp.last_match[:v_type]
        {}.tap do |hash|
          value.each do |k, v|
            hash[_deserialize(k_type, k)] = _deserialize(v_type, v)
          end
        end
      when :Object
        value
      else # model
        _model = HelixWebServices.const_get(type).new
        _model.build_from_hash(value)
      end
    end

    def to_s
      to_hash.to_s
    end

    # to_body is an alias to to_body (backward compatibility))
    def to_body
      to_hash
    end

    # return the object in the form of hash
    def to_hash
      hash = {}
      self.class.attribute_map.each_pair do |attr, param|
        value = self.send(attr)
        next if value.nil?
        hash[param] = _to_hash(value)
      end
      hash
    end

    # Method to output non-array value in the form of hash
    # For object, use to_hash. Otherwise, just return the value
    def _to_hash(value)
      if value.is_a?(Array)
        value.compact.map{ |v| _to_hash(v) }
      elsif value.is_a?(Hash)
        {}.tap do |hash|
          value.each { |k, v| hash[k] = _to_hash(v) }
        end
      elsif value.respond_to? :to_hash
        value.to_hash
      else
        value
      end
    end

  end
end
